searchData={"items":[{"type":"module","title":"ecron","doc":"`ecron` enables dynamic scheduling of tasks using crontab-style expressions. \nIt provides a flexible API for managing recurring jobs, one-time tasks, \nand time-based message delivery.","ref":"ecron.html"},{"type":"function","title":"ecron.activate/1","doc":"Activates a previously deactivated job from the default registry. \nThe job can be deactivated using [`deactivate/1`](#deactivate/1).","ref":"ecron.html#activate/1"},{"type":"function","title":"ecron.activate/2","doc":"Activates a previously deactivated job from the specified registry. \nThe job will resume from the current time.\nThe job can be deactivated using [`deactivate/2`](#deactivate/2).\n\nParameters:\n* `Register` - Process name where job is registered\n* `JobName` - Name of job to activate\n\nReturns: `ok` | `{error, not_found}`","ref":"ecron.html#activate/2"},{"type":"function","title":"ecron.add/3","doc":"Same as [`add(ecron_local, JobName, Spec, MFA)`](`add/4`).\n`ecron_local` is the default register name, always exists.","ref":"ecron.html#add/3"},{"type":"function","title":"Examples - ecron.add/3","doc":"<!-- tabs-open -->","ref":"ecron.html#add/3-examples"},{"type":"function","title":"Erlang - ecron.add/3","doc":"```erlang\nJobName = every_4am_job,\nMFA = {io, format, [\"Run at 04:00 everyday.~n\", []]},\n{ok, JobName} = ecron:add(JobName, \"0 4 * * *\", MFA),\necron:statistic(JobName).\n```","ref":"ecron.html#add/3-erlang"},{"type":"function","title":"Elixir - ecron.add/3","doc":"```elixir\njob_name = :every_4am_job\nmfa = {IO, :puts, [\"Run at 04:00 everyday.\\n\"]}\n{:ok, ^job_name} = :ecron.add(job_name, \"0 4 * * *\", mfa)\n:ecron.statistic(job_name)\n```\n\n<!-- tabs-close -->","ref":"ecron.html#add/3-elixir"},{"type":"function","title":"ecron.add/4","doc":"Same as [`add(Register, JobName, Spec, MFA, unlimited, unlimited, [])`](`add/7`).","ref":"ecron.html#add/4"},{"type":"function","title":"Examples - ecron.add/4","doc":"<!-- tabs-open -->","ref":"ecron.html#add/4-examples"},{"type":"function","title":"Erlang - ecron.add/4","doc":"```erlang\nRegister = my_cronjob_register,\n{ok, _Pid} = ecron:start_link(Register, []),\nJobName = every_4am_job,\nMFA = {io, format, [\"Run at 04:00 everyday.~n\", []]},\n{ok, JobName} = ecron:add(Register, JobName, \"0 4 * * *\", MFA),\necron:statistic(Register, JobName).\n```","ref":"ecron.html#add/4-erlang"},{"type":"function","title":"Elixir - ecron.add/4","doc":"```elixir\nregister = :my_cronjob_register\n{:ok, _Pid} = :ecron.start_link(register, [])\njob_name = :every_4am_job\nmfa = {IO, :puts, [\"Run at 04:00 everyday.\\n\"]}\n{:ok, ^job_name} = :ecron.add(register, job_name, \"0 4 * * *\", mfa)\n:ecron.statistic(register, job_name)\n```\n<!-- tabs-close -->","ref":"ecron.html#add/4-elixir"},{"type":"function","title":"ecron.add/6","doc":"Same as [`add(ecron_local, JobName, Spec, MFA, Start, End, Opts)`](`add/7`).","ref":"ecron.html#add/6"},{"type":"function","title":"ecron.add/7","doc":"Adds a new crontab job with specified parameters. \nJobs exceeding their limits are automatically removed.\n\nParameters:\n* `Register` - The process name where the job will be registered\n* `JobName` - Unique identifier for the job. Returns `{error, already_exist}` if duplicate\n* `Spec` - [Crontab expression](readme.html#cron-expression-guide) defining execution schedule\n* `MFA` - `{Module, Function, Args}` to execute when triggered\n* `Start` - Start time `{Hour,Min,Sec}` or `unlimited` for immediate start\n* `End` - End time `{Hour,Min,Sec}` or `unlimited` for no end\n* `Opts` - Options list:\n    * `{singleton, boolean()}` - If true (default), prevents concurrent execution\n    * `{max_count, pos_integer() | unlimited}` - Maximum executions allowed\n    * `{max_runtime_ms, pos_integer() | unlimited}` - Maximum runtime milliseconds allowed for each execution\n\nReturns: `{ok, JobName}` | `{error, already_exist}` | `{error, parse_error(), term()}`\n\n# Examples\n<!-- tabs-open -->","ref":"ecron.html#add/7"},{"type":"function","title":"Erlang - ecron.add/7","doc":"```erlang\nRegister = my_cronjob_register,\n{ok, _Pid} = ecron:start_link(Register, []),\nJobName = every_4am_job,\nMFA = {io, format, [\"Run at 04:00 everyday.~n\", []]},\n{ok, JobName} = ecron:add(Register, JobName, \"0 4 * * *\", MFA, {1, 0, 0}, unlimited, [{singleton, true}]),\necron:statistic(Register, JobName).\n```","ref":"ecron.html#add/7-erlang"},{"type":"function","title":"Elixir - ecron.add/7","doc":"```elixir\nregister = :my_cronjob_register\n{:ok, _Pid} = :ecron.start_link(register, [])\njob_name = :every_4am_job\nmfa = {IO, :puts, [\"Run at 04:00 everyday.\\n\"]}\n{:ok, ^job_name} = :ecron.add(register, job_name, \"0 4 * * *\", mfa, {1, 0, 0}, :unlimited, [singleton: true])\n:ecron.statistic(register, job_name)\n```\n\n<!-- tabs-close -->\n> #### Singleton {: .info}\n> When a job's singleton option is set to true, the system checks \n> if there is already an instance of the job running before starting a new execution. \n> If an instance is already running（old pid is alive）, the new execution will be skipped.","ref":"ecron.html#add/7-elixir"},{"type":"function","title":"ecron.add_with_count/3","doc":"Same as [`add_with_count(ecron_local, make_ref(), Spec, MFA, RunCount)`](`add_with_count/5`).\n`ecron_local` is the default register name, always exists.","ref":"ecron.html#add_with_count/3"},{"type":"function","title":"Examples - ecron.add_with_count/3","doc":"The job will be auto deleted after running 10 times.\n<!-- tabs-open -->","ref":"ecron.html#add_with_count/3-examples"},{"type":"function","title":"Erlang - ecron.add_with_count/3","doc":"```erlang\nJobName = every_4am_job,\nMFA = {io, format, [\"Run at 04:00 everyday.~n\", []]},\n{ok, JobName} = ecron:add_with_count(JobName, \"0 4 * * *\", MFA, 10),\necron:statistic(JobName).\n```","ref":"ecron.html#add_with_count/3-erlang"},{"type":"function","title":"Elixir - ecron.add_with_count/3","doc":"```elixir\njob_name = :every_4am_job\nmfa = {IO, :puts, [\"Run at 04:00 everyday.\\n\"]}\n{:ok, ^job_name} = :ecron.add_with_count(job_name, \"0 4 * * *\", mfa, 10)\n:ecron.statistic(job_name)\n```\n\n<!-- tabs-close -->","ref":"ecron.html#add_with_count/3-elixir"},{"type":"function","title":"ecron.add_with_count/5","doc":"Same as [`add(Register, JobName, Spec, MFA, unlimited, unlimited, [{max_count, RunCount}])`](`add/7`).","ref":"ecron.html#add_with_count/5"},{"type":"function","title":"Examples - ecron.add_with_count/5","doc":"The job will be auto deleted after running 10 times.\n<!-- tabs-open -->","ref":"ecron.html#add_with_count/5-examples"},{"type":"function","title":"Erlang - ecron.add_with_count/5","doc":"```erlang\nRegister = my_cronjob_register,\n{ok, _Pid} = ecron:start_link(Register, []),\nJobName = every_4am_job,\nMFA = {io, format, [\"Run at 04:00 everyday.~n\", []]},\n{ok, JobName} = ecron:add_with_count(Register, JobName, \"0 4 * * *\", MFA, 10),\necron:statistic(Register, JobName).\n```","ref":"ecron.html#add_with_count/5-erlang"},{"type":"function","title":"Elixir - ecron.add_with_count/5","doc":"```elixir\nregister = :my_cronjob_register\n{:ok, _Pid} = :ecron.start_link(register, [])\njob_name = :every_4am_job\nmfa = {IO, :puts, [\"Run at 04:00 everyday.\\n\"]}\n{:ok, ^job_name} = :ecron.add_with_count(register, job_name, \"0 4 * * *\", mfa, 10)\n:ecron.statistic(register, job_name)\n```\n\n<!-- tabs-close -->","ref":"ecron.html#add_with_count/5-elixir"},{"type":"function","title":"ecron.add_with_time/5","doc":"Add a job into default register with start and end time.\nIf you want to use the specified register, use [`add_with_time/6`](#add_with_time/6).","ref":"ecron.html#add_with_time/5"},{"type":"function","title":"Examples - ecron.add_with_time/5","doc":"The job will be auto skipped if the current time is not between 04:00 and 12:00 everyday.\n<!-- tabs-open -->","ref":"ecron.html#add_with_time/5-examples"},{"type":"function","title":"Erlang - ecron.add_with_time/5","doc":"```erlang\nJobName = on_hour_job,\nMFA = {io, format, [\"Run at 04:00-12:00 on the hour.~n\", []]},\n{ok, JobName} = ecron:add_with_time(JobName, \"0 1-12/1 * * *\", MFA, {4, 0, 0}, {12, 0, 0}),\necron:statistic(JobName).\n```","ref":"ecron.html#add_with_time/5-erlang"},{"type":"function","title":"Elixir - ecron.add_with_time/5","doc":"```elixir\njob_name = :on_hour_job\nmfa = {IO, :puts, [\"Run at 04:00-12:00 on the hour.\\n\"]}\n{:ok, ^job_name} = :ecron.add_with_time(job_name, \"0 1-12/1 * * *\", mfa, {4, 0, 0}, {12, 0, 0})\n:ecron.statistic(job_name)\n```\n<!-- tabs-close -->\n    \n> #### TimeRange {: .info}\n> The start time must be less than the maximum value in the spec, and the end time must be greater than the minimum value in the spec.\n>\n> For example: With spec `0 1-12/1 * * *` the max value is 12 and min value is 1,\n> so start time must be less than {12,0,0} and end time must be greater than {1,0,0}, with start < end.\n    \nIf we can not find the next schedule time in the next 5 years, return `cant find next schedule time in the next 5 years`.\n    \n```erlang\necron:add_with_time(invalid_job, \"* 0,13 * * *\", {io, format, [\"test\"]},{1,0,0},{12,0,0}).\n{error,invalid_time,\n        #{reason => \"cant find next schedule time in the next 5 years\",\n            start => {1,0,0},\n            stop => {12,0,0},\n            spec => \"* 0,13 * * *\"}}\n```","ref":"ecron.html#add_with_time/5-elixir"},{"type":"function","title":"ecron.add_with_time/6","doc":"Add a job into specified register with start and end time.\nIf you want to use the default register, use [`add_with_time/5`](#add_with_time/5).","ref":"ecron.html#add_with_time/6"},{"type":"function","title":"Examples - ecron.add_with_time/6","doc":"The job will be auto skipped if the current time is not between 04:00 and 12:00 everyday.\n<!-- tabs-open -->","ref":"ecron.html#add_with_time/6-examples"},{"type":"function","title":"Erlang - ecron.add_with_time/6","doc":"```erlang\nRegister = my_cronjob_register,\n{ok, _Pid} = ecron:start_link(Register, []),\nJobName = on_hour_job,\nMFA = {io, format, [\"Run at 04:00-12:00 on the hour.~n\", []]},\n{ok, JobName} = ecron:add_with_time(Register, JobName, \"0 1-12/1 * * *\", MFA, {4, 0, 0}, {12, 0, 0}),\necron:statistic(Register, JobName).\n```","ref":"ecron.html#add_with_time/6-erlang"},{"type":"function","title":"Elixir - ecron.add_with_time/6","doc":"```elixir\nregister = :my_cronjob_register\n{:ok, _Pid} = :ecron.start_link(register, [])\njob_name = :on_hour_job\nmfa = {IO, :puts, [\"Run at 04:00-12:00 on the hour.\\n\"]}\n{:ok, ^job_name} = :ecron.add_with_time(register, job_name, \"0 1-12/1 * * *\", mfa, {4, 0, 0}, {12, 0, 0})\n:ecron.statistic(register, job_name)\n```\n<!-- tabs-close -->\n\n> #### TimeRange {: .info}\n> The start time must be less than the maximum value in the spec, and the end time must be greater than the minimum value in the spec.\n>\n> For example: With spec `0 1-12/1 * * *` the max value is 12 and min value is 1,\n> so start time must be less than {12,0,0} and end time must be greater than {1,0,0}, with start < end.\n\nIf we can not find the next schedule time in the next 5 years, return `cant find next schedule time in the next 5 years`.\n\n```erlang\necron:add_with_time(invalid_job, \"* 0,13 * * *\", {io, format, [\"test\"]},{1,0,0},{12,0,0}).\n{error,invalid_time,\n       #{reason => \"cant find next schedule time in the next 5 years\",\n         start => {1,0,0},\n         stop => {12,0,0},\n         spec => \"* 0,13 * * *\"}}\n```","ref":"ecron.html#add_with_time/6-elixir"},{"type":"function","title":"ecron.create/3","doc":"Same as [`create(JobName, Spec, MFA, #{})`](`create/4`).","ref":"ecron.html#create/3"},{"type":"function","title":"Examples - ecron.create/3","doc":"<!-- tabs-open -->","ref":"ecron.html#create/3-examples"},{"type":"function","title":"Erlang - ecron.create/3","doc":"```erlang\nUniqueJobName = every_4am_job,\nMFA = {io, format, [\"Run at 04:00 everyday.~n\", []]},\n{ok, UniqueJobName} = ecron:create(UniqueJobName, \"0 4 * * *\", MFA),\necron:statistic(UniqueJobName).\n```","ref":"ecron.html#create/3-erlang"},{"type":"function","title":"Elixir - ecron.create/3","doc":"```elixir\njob_name = :every_4am_job\nmfa = {IO, :puts, [\"Run at 04:00 everyday.\\n\"]}\n{:ok, ^job_name} = :ecron.create(job_name, \"0 4 * * *\", mfa)\n:ecron.statistic(job_name)\n```\n\n<!-- tabs-close -->","ref":"ecron.html#create/3-elixir"},{"type":"function","title":"ecron.create/4","doc":"Adds a new crontab job with specified parameters. \nJobs exceeding their limits are automatically removed.\n\nParameters:\n* `JobName` - Unique identifier for the job. Returns `{error, already_exist}` if duplicate\n* `Spec` - [Crontab expression](readme.html#cron-expression-guide) defining execution schedule\n* `MFA` - `{Module, Function, Args}` to execute when triggered\n* `Opts` - Options map:\n   * register: `atom()` - The process name where the job will be registered (default: `ecron_local`)\n   * start_time: `{Hour,Min,Sec}|unlimited` - Start time for job execution (default: `unlimited`)\n   * end_time: `{Hour,Min,Sec}|unlimited` - End time for job execution (default: `unlimited`) \n   * singleton: `boolean()` - If true, prevents concurrent execution (default: `false`)\n   * max_count: `pos_integer()|unlimited` - Maximum number of executions allowed (default: `unlimited`). It will be automatically removed When a job reaches its max_count limit.\n   * max_runtime_ms: `pos_integer()|unlimited` - Maximum runtime in milliseconds per execution (default: `unlimited`)\n\nReturns: `{ok, JobName}` | `{error, already_exist}` | `{error, parse_error(), term()}`\n\n# Examples\n<!-- tabs-open -->","ref":"ecron.html#create/4"},{"type":"function","title":"Erlang - ecron.create/4","doc":"```erlang\nRegister = my_job_register,\n{ok, _Pid} = ecron:start_link(Register, []),\nJobName = every_4am_job,\nMFA = {io, format, [\"Run at 04:00 everyday.~n\", []]},\n{ok, JobName} = ecron:create(JobName, \"0 4 * * *\", MFA, #{\n    start_time => {1, 0, 0}, \n    end_time => unlimited, \n    singleton => false,\n    max_count => unlimited,\n    max_runtime_ms => 1000,\n    register => Register\n}),\necron:statistic(Register, JobName).\n```","ref":"ecron.html#create/4-erlang"},{"type":"function","title":"Elixir - ecron.create/4","doc":"```elixir\nregister = :my_job_register\n{:ok, _Pid} = :ecron.start_link(register, [])\njob_name = :every_4am_job\nmfa = {IO, :puts, [\"Run at 04:00 everyday.\\n\"]}\n{:ok, ^job_name} = :ecron.create(job_name, \"0 4 * * *\", mfa, %{\n    :start_time => {1, 0, 0}, \n    :end_time => :unlimited, \n    :singleton => false,\n    :max_count => :unlimited,\n    :max_runtime_ms => 1000,\n    :register => register\n})\n:ecron.statistic(register, job_name)\n```\n\n<!-- tabs-close -->\n> #### Singleton {: .info}\n> When a job's singleton option is set to true, the system checks \n> if there is already an instance of the job running before starting a new execution. \n> If an instance is already running (old pid is alive), the new execution will be skipped.\n\n> #### TimeRange {: .info}\n> The start time must be less than the maximum value in the spec, and the end time must be greater than the minimum value in the spec.\n>\n> For example: With spec `0 1-12/1 * * *` the max value is 12 and min value is 1,\n> so start time must be less than {12,0,0} and end time must be greater than {1,0,0}, with start < end.\n\nIf we can not find the next schedule time in the next 5 years, return 'cant find next schedule time in the next 5 years'.\n\n```erlang\n ecron:create(invalid_job, \"* 0,13 * * *\", {io, format, [test]}, #{\n       start_time => {1,0,0},\n       end_time => {12,0,0}\n   }).\n{error,invalid_time,\n       #{reason => \"cant find next schedule time in the next 5 years\",\n         spec => \"* 0,13 * * *\",\n         start_time => {1,0,0},\n         end_time => {12,0,0}}}\n```","ref":"ecron.html#create/4-elixir"},{"type":"function","title":"ecron.deactivate/1","doc":"Deactivates an existing job temporarily from the default registry. \nThe job can be reactivated using [`activate/1`](#activate/1).","ref":"ecron.html#deactivate/1"},{"type":"function","title":"ecron.deactivate/2","doc":"Deactivates an existing job temporarily from the specified registry. \nThe job can be reactivated using [`activate/2`](#activate/2).\n\nParameters:\n* `Register` - Process name where job is registered  \n* `JobName` - Name of job to deactivate\n\nReturns: `ok` | `{error, not_found}`","ref":"ecron.html#deactivate/2"},{"type":"function","title":"ecron.delete/1","doc":"Deletes an existing job from the default registry. \nIf the job does not exist, it will be ignored.\nUse [`delete/2`](#delete/2) to delete a job from a specified registry.\n\nReturns: `ok`","ref":"ecron.html#delete/1"},{"type":"function","title":"ecron.delete/2","doc":"Deletes an existing job from the specified registry.\nIf the job does not exist, it will be ignored.\nUse [`delete/1`](#delete/1) to delete a job from the default registry.\n\nParameters:\n* `Register` - Process name where job is registered\n* `JobName` - Name of job to delete\n\nReturns: `ok`","ref":"ecron.html#delete/2"},{"type":"function","title":"ecron.parse_spec/2","doc":"Parses a crontab specification and returns the next N trigger times.\nUseful for debugging and validating crontab expressions.\n\nParameters:\n* `Spec` - Crontab expression to parse\n* `Num` - Number of future trigger times to calculate\n\nReturns: `{ok, #{type => cron|every, crontab => parsed_spec, next => [rfc3339_string()]}}` |\n`{error, parse_error(), term()}`","ref":"ecron.html#parse_spec/2"},{"type":"function","title":"ecron.reload/0","doc":"Reloads all tasks manually. Useful when system time has changed significantly.\n\nThis will:\n- Recalculate next execution times for all jobs\n- Reset internal timers\n- Apply to both default and global registries\n\nReturns: `ok`","ref":"ecron.html#reload/0"},{"type":"function","title":"ecron.reload/1","doc":"Reloads all tasks manually. Useful when system time has changed significantly.\n\nParameters:\n* `Register` - Process name where job is registered\n\nReturns: `ok`","ref":"ecron.html#reload/1"},{"type":"function","title":"ecron.send_after/3","doc":"Create a one-time timer that sends a message when the crontab spec is triggered.\n\nParameters:\n* `Spec` - Crontab expression defining when to trigger\n* `Dest` - Destination pid() or registered name to receive message\n* `Message` - Term to send when timer triggers\n\nNotes:\n* Similar to `erlang:send_after/3` but uses crontab format\n* Dest pid must be local\n* Maximum time value is 4294967295 milliseconds\n* Timer auto-cancels if destination process dies\n\nReturns: `{ok, reference()}` | `{error, parse_error(), term()}`\n\n# Examples\n<!-- tabs-open -->","ref":"ecron.html#send_after/3"},{"type":"function","title":"Erlang - ecron.send_after/3","doc":"```erlang\necron:send_after(\"*/3 * * * * *\", self(), hello_world).\n```","ref":"ecron.html#send_after/3-erlang"},{"type":"function","title":"Elixir - ecron.send_after/3","doc":"```elixir\n:ecron.send_after(\"*/3 * * * * *\", self(), :hello_world)\n```\n<!-- tabs-close -->\n\n> #### Statistic {: .info}\n> This is one-time timer, so it not seen in `statistic/0` result.    \n>\n> Use `erlang:cancel_timer/1` to cancel, not `ecron:delete/1`","ref":"ecron.html#send_after/3-elixir"},{"type":"function","title":"ecron.send_interval/2","doc":"Sends a message to a process repeatedly based on a crontab schedule from the default registry. \nSame as [`send_interval(Spec, Pid, Message, #{register => ecron_local})`](`send_interval/4`).","ref":"ecron.html#send_interval/2"},{"type":"function","title":"ecron.send_interval/3","doc":"Sends a message to a process repeatedly based on a crontab schedule from the default registry. \nSame as [`send_interval(ecron_local, make_ref(), Spec, Pid, Message, unlimited, unlimited, [])`](`send_interval/7`).","ref":"ecron.html#send_interval/3"},{"type":"function","title":"ecron.send_interval/4","doc":"Evaluates `Pid ! Message` repeatedly after crontab schedule milliseconds.\n\nParameters:\n* `Spec` - Crontab expression defining when to trigger\n* `Pid` - Destination process ID or registered name\n* `Message` - Term to send on each trigger\n* `Opts` - Same options as `create/4`\n\nReturns: `{ok, reference()}` | `{error, parse_error(), term()}`\n\n# Examples\n<!-- tabs-open -->","ref":"ecron.html#send_interval/4"},{"type":"function","title":"Erlang - ecron.send_interval/4","doc":"```erlang\necron:send_interval(\"*/3 * * * * *\", self(), hello_world).\n```","ref":"ecron.html#send_interval/4-erlang"},{"type":"function","title":"Elixir - ecron.send_interval/4","doc":"```elixir\n:ecron.send_interval(\"*/3 * * * * *\", self(), :hello_world)\n```\n<!-- tabs-close -->\n\n> #### Statistic {: .info}\n> This is repeatable timer, so it seen in `statistic/0` result.    \n>\n> Use `ecron:delete/1` to cancel, not `erlang:cancel_timer/1`","ref":"ecron.html#send_interval/4-elixir"},{"type":"function","title":"ecron.send_interval/5","doc":"Sends a message to a process repeatedly based on a crontab schedule from the specified registry. \nSame as [`send_interval(Register, Name, Spec, Pid, Message, unlimited, unlimited, [])`](`send_interval/7`).","ref":"ecron.html#send_interval/5"},{"type":"function","title":"ecron.send_interval/7","doc":"Same as [`send_interval(register(), name(), Spec, self(), Message, Start, End, Option)`](`send_interval/8`).","ref":"ecron.html#send_interval/7"},{"type":"function","title":"ecron.send_interval/8","doc":"Sends a message to a process repeatedly based on a crontab schedule.","ref":"ecron.html#send_interval/8"},{"type":"function","title":"Parameters - ecron.send_interval/8","doc":"* `Register` - Process name where job will be registered\n* `JobName` - Unique identifier for the job\n* `Spec` - Crontab expression defining execution schedule\n* `Pid` - Destination process ID or registered name\n* `Message` - Term to send on each trigger\n* `Start` - Start time `{Hour,Min,Sec}` or `unlimited`\n* `End` - End time `{Hour,Min,Sec}` or `unlimited`\n* `Option` - Same options as `add/7`\n\nReturns: `{ok, reference()}` | `{error, parse_error(), term()}`","ref":"ecron.html#send_interval/8-parameters"},{"type":"function","title":"ecron.start_link/1","doc":"Create a new registry.\nSame as [`start_link(Register, [])`](#start_link/2).\n\nParameters:\n* `Register` - Process name where job is registered\n\nReturns: `{ok, Pid}` | `{error, Reason}`","ref":"ecron.html#start_link/1"},{"type":"function","title":"ecron.start_link/2","doc":"Starts an new registry.\n\nParameters:\n* `Name` - Process name where job is registered\n* `JobSpec` - Crontab expression to parse list\n\nReturns: `{ok, Pid}` | `{error, Reason}`","ref":"ecron.html#start_link/2"},{"type":"function","title":"ecron.statistic/0","doc":"Retrieves statistics for both default and global registered jobs.\n\nReturns: List of statistics for all jobs in both local and global registries.\nEach statistic entry contains the same information as `statistic/2`.\n\n```erlang\nUniqueJobName = every_4am_job,\nMFA = {io, format, [\"Run at 04:00 everyday.~n\", []]},\n{ok, UniqueJobName} = ecron:create(UniqueJobName, \"0 4 * * *\", MFA),\necron:statistic(UniqueJobName).\n\n[#{name => every_4am_job,node => nonode@nohost,ok => 0,\n   status => activate,type => cron,\n   next =>\n       [\"2025-02-19T04:00:00+08:00\",\"2025-02-20T04:00:00+08:00\",\n        \"2025-02-21T04:00:00+08:00\",\"2025-02-22T04:00:00+08:00\",\n        ...\n        ],\n   opts => [{singleton,false}, {max_count,unlimited}, {max_runtime_ms,unlimited}],\n   mfa => {io,format,[\"Run at 04:00 everyday.~n\",[]]},\n   aborted => 0,crashed => 0,skipped => 0, run_microsecond => [],\n   start_time => {0,0,0},\n   end_time => {23,59,59},\n   crontab =>\n       #{second => [0],\n         month => '*',\n         minute => [0],\n         hour => [4],\n         day_of_month => '*',day_of_week => '*'},\n   results => []}]\n```","ref":"ecron.html#statistic/0"},{"type":"function","title":"ecron.statistic/1","doc":"Retrieves statistics for default registry.\nUse [`statistic/2`](#statistic/2) to get statistics for a specific registry.\n\nReturns: List of statistics for all jobs in local registries.\nEach statistic entry contains the same information as `statistic/2`.","ref":"ecron.html#statistic/1"},{"type":"function","title":"ecron.statistic/2","doc":"Retrieves statistics for a specific job from the specified registry.\nUse [`statistic/1`](#statistic/1) to get statistics for the default registry.\n\nParameters:\n* `Register` - Process name where job is registered\n* `JobName` - Name of job to get statistics for\n\nReturns: `{ok, statistic()}` | `{error, not_found}`\n\nWhere statistic() contains:\n- Job configuration\n- Execution counts (ok/crashed/aborted/skipped)  \n- Latest results\n- Run times\n- Next scheduled runs\n```erlang\n1> ecron:statistic(basic).\n[#{name => basic,node => nonode@nohost,ok => 0,\n   status => activate,type => cron,\n   next => [\"2025-02-20T22:15:00+08:00\"...],        \n   opts => [{singleton,false},{max_count,unlimited},{max_runtime_ms,unlimited}],\n   mfa => {io,format,['Runs on 0, 15, 30, 45 minutes~n']},\n   aborted => 0,crashed => 0,skipped => 0,\n   start_time => {0,0,0},\n   end_time => {23,59,59},\n   run_microsecond => [],\n   crontab =>\n       #{second => [0],\n         month => '*',\n         minute => [0,15,30,45],\n         hour => '*',day_of_month => '*',day_of_week => '*'},\n   results => []}]\n```\n- `ok`: successful job.\n- `crashed`: crashed job.\n- `skipped`: singleton(true) job has been skipped due to the previous task still running.\n- `aborted`: job has been aborted due to exceeding `max_runtime_ms`.","ref":"ecron.html#statistic/2"},{"type":"type","title":"ecron.crontab/0","doc":"","ref":"ecron.html#t:crontab/0"},{"type":"type","title":"ecron.crontab_spec/0","doc":"","ref":"ecron.html#t:crontab_spec/0"},{"type":"type","title":"ecron.ecron_result/0","doc":"","ref":"ecron.html#t:ecron_result/0"},{"type":"type","title":"ecron.end_at/0","doc":"","ref":"ecron.html#t:end_at/0"},{"type":"type","title":"ecron.mfargs/0","doc":"","ref":"ecron.html#t:mfargs/0"},{"type":"type","title":"ecron.name/0","doc":"","ref":"ecron.html#t:name/0"},{"type":"type","title":"ecron.option_list/0","doc":"","ref":"ecron.html#t:option_list/0"},{"type":"type","title":"ecron.options/0","doc":"","ref":"ecron.html#t:options/0"},{"type":"type","title":"ecron.parse_error/0","doc":"","ref":"ecron.html#t:parse_error/0"},{"type":"type","title":"ecron.register/0","doc":"","ref":"ecron.html#t:register/0"},{"type":"type","title":"ecron.rfc3339_string/0","doc":"","ref":"ecron.html#t:rfc3339_string/0"},{"type":"type","title":"ecron.start_at/0","doc":"","ref":"ecron.html#t:start_at/0"},{"type":"type","title":"ecron.statistic/0","doc":"","ref":"ecron.html#t:statistic/0"},{"type":"extras","title":"Home","doc":"# Ecron [![GitHub Actions](https://github.com/zhongwencool/ecron/actions/workflows/ci.yml/badge.svg)](https://github.com/zhongwencool/ecron) [![CodeCov](https://codecov.io/gh/zhongwencool/ecron/branch/master/graph/badge.svg?token=FI9WAQ6UG5)](https://codecov.io/gh/zhongwencool/ecron) [![Hex](https://img.shields.io/hexpm/v/ecron.svg?style=flat)](https://hex.pm/packages/ecron) [![Tag](https://img.shields.io/github/tag/zhongwencool/ecron.svg)](https://img.shields.io/github/tag/zhongwencool/ecron.svg) [![License](https://img.shields.io/hexpm/l/ecron.svg)](https://img.shields.io/hexpm/l/ecron.svg) [![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ecron/)\r\n\r\n> Resilient, lightweight, efficient, cron-like job scheduling library for Erlang/Elixir.\r\n\r\n![EcronLogo](https://github.com/user-attachments/assets/ad3b493b-c1ef-4528-b7be-1e074c17b993)\r\n\r\nEcron supports both cron-style and interval-based job scheduling, focusing on resiliency, correctness, and lightweight with comprehensive testing via PropTest.","ref":"readme.html"},{"type":"extras","title":"Use Case - Home","doc":"Ecron's precise scheduling is perfect for:\r\n\r\n- **Security**: `0 3 * * 0` Rotate API keys and dynamic credentials automatically every Sunday at 3 AM\r\n- **Flash Sale**: `0 8 * * *` Launch flash sales with precision at 8 AM\r\n- **Analytics**: `0 9 * * 1` Sending comprehensive weekly reports every Monday at 9 AM\r\n- **Disk Protection**: `30 23 * * *` Compress and archive old logs at 23:30 daily\r\n- **Data Cleanup**: `0 1 1 * *` Pruning inactive users on the first day of each month\r\n- **Data Backup**: `0 2 * * *` Create reliable Mnesia database backups every day at 2 AM","ref":"readme.html#use-case"},{"type":"extras","title":"Setup - Home","doc":"<!-- tabs-open -->","ref":"readme.html#setup"},{"type":"extras","title":"Erlang - Home","doc":"```erlang\r\n  %% rebar.config\r\n  {deps, [{ecron, \"~> 1.1.0\"}]}\r\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - Home","doc":"```elixir\r\n  # mix.exs\r\n  def deps do\r\n    [{:ecron, \"~> 1.1.0\"}]\r\n  end\r\n```\r\n<!-- tabs-close -->","ref":"readme.html#elixir"},{"type":"extras","title":"Configuration Usage - Home","doc":"<!-- tabs-open -->","ref":"readme.html#configuration-usage"},{"type":"extras","title":"Erlang - Home","doc":"Configure ecron in your `sys.config` file with job specifications:\r\n\r\n```erlang\r\n%% sys.config\r\n[\r\n   {ecron, [\r\n      {local_jobs, [\r\n         %% {JobName, CrontabSpec, {M, F, A}}\r\n         %% {JobName, CrontabSpec, {M, F, A}, PropListOpts}\r\n         %% CrontabSpec\r\n            %%  1. \"Minute Hour DayOfMonth Month DayOfWeek\"\r\n            %%  2. \"Second Minute Hour DayOfMonth Month DayOfWeek\"\r\n            %%  3. @yearly | @annually | @monthly | @weekly | @daily | @midnight | @hourly\r\n            %%  4. @every 1h2m3s\r\n                  \r\n         {basic, \"*/15 * * * *\", {io, format, [\"Runs on 0, 15, 30, 45 minutes~n\"]}},\r\n         {sec_in_spec, \"0 0 1-6/2,18 * * *\", {io, format, [\"Runs on 1,3,6,18 o'clock:~n\"]}},\r\n         {hourly, \"@hourly\", {io, format, [\"Runs every(0-23) o'clock~n\"]}},    \r\n         {interval, \"@every 30m\", {io, format, [\"Runs every 30 minutes\"]}},         \r\n         {limit_time, \"*/15 * * * *\", {io, format, [\"Runs 0, 15, 30, 45 minutes after 8:20am~n\"]}, [{start_time, {8,20,0}}, {end_time, {23, 59, 59}}]},\r\n         {limit_count, \"@every 1m\", {io, format, [\"Runs 10 times\"]}, [{max_count, 10}]},         \r\n         {limit_concurrency, \"@minutely\", {timer, sleep, [61000]}, [{singleton, true}]},         \r\n         {limit_runtime_ms, \"@every 1m\", {timer, sleep, [2000]}, [{max_runtime_ms, 1000}]}\r\n     ]}     \r\n    }\r\n].\r\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - Home","doc":"Configure ecron in your `config.exs` file with job specifications:\r\n```elixir\r\n# config/config.exs\r\nconfig :ecron,  \r\n  local_jobs: [\r\n    # {job_name, crontab_spec, {module, function, args}}\r\n    # {job_name, crontab_spec, {module, function, args}, PropListOpts}\r\n    # CrontabSpec formats:\r\n    #  1. \"Minute Hour DayOfMonth Month DayOfWeek\"\r\n    #  2. \"Second Minute Hour DayOfMonth Month DayOfWeek\"\r\n    #  3. @yearly | @annually | @monthly | @weekly | @daily | @midnight | @hourly\r\n    #  4. @every 1h2m3s\r\n    \r\n    {:basic, \"*/15 * * * *\", {IO, :puts, [\"Runs on 0, 15, 30, 45 minutes\"]}},\r\n    {:sec_in_spec, \"0 0 1-6/2,18 * * *\", {IO, :puts, [\"Runs on 1,3,6,18 o'clock:\"]}},\r\n    {:hourly, \"@hourly\", {IO, :puts, [\"Runs every(0-23) o'clock\"]}},\r\n    {:interval, \"@every 30m\", {IO, :puts, [\"Runs every 30 minutes\"]}},    \r\n    {:limit_time, \"*/15 * * * *\", {IO, :puts, [\"Runs 0, 15, 30, 45 minutes after 8:20am\"]}, [start_time: {8,20,0}, end_time: {23, 59, 59}]},\r\n    {:limit_count, \"@every 1m\", {IO, :puts, [\"Runs 10 times\"]}, [max_count: 10]},\r\n    {:limit_concurrency, \"@minutely\", {Process, :sleep, [61000]}, [singleton: true]},\r\n    {:limit_runtime_ms, \"@every 1m\", {Process, :sleep, [2000]}, [max_runtime_ms: 1000]}\r\n  ] \r\n```\r\n<!-- tabs-close -->\r\n\r\n* When a job reaches its `max_count` limit, it will be automatically removed. By default, `max_count` is set to `unlimited`.\r\n* By default, `singleton` is set to `false`, which means multiple instances of the same job can run concurrently. Set `singleton` to `true` to ensure only one instance runs at a time.\r\n\r\nFor all PropListOpts, refer to the documentation for [`ecron:create/4`](https://hexdocs.pm/ecron/ecron.html#create/4).","ref":"readme.html#elixir"},{"type":"extras","title":"Runtime Usage - Home","doc":"Besides loading jobs from config files at startup, you can add jobs from your code.\r\n<!-- tabs-open -->","ref":"readme.html#runtime-usage"},{"type":"extras","title":"Erlang - Home","doc":"```erlang\r\nJobName = every_4am_job,\r\nMFA = {io, format, [\"Run at 04:00 every day.\\n\"]},\r\nOptions = #{max_runtime_ms => 1000},\r\necron:create(JobName, \"0 4 * * *\", MFA, Options).\r\nStatistic = ecron:statistic(JobName),\r\necron:delete(JobName),\r\nStatistic.\r\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - Home","doc":"```elixir\r\njob_name = :every_4am_job\r\nmfa = {IO, :puts, [\"Run at 04:00 every day.\\n\"]}\r\noptions = %{max_runtime_ms: 1000}\r\n{:ok, ^job_name} = :ecron.create(job_name, \"0 4 * * *\", mfa, options)\r\nstatistic = :ecron.statistic(job_name)\r\n:ecron.delete(job_name)\r\nstatistic\r\n```\r\n<!-- tabs-close -->","ref":"readme.html#elixir"},{"type":"extras","title":"Multi Register - Home","doc":"For most applications, the above two methods are enough. However, Ecron offers a more flexible way to manage job lifecycles.\r\n\r\nFor example, when applications A and B need separate cron jobs, you can create a dedicated register for each. This ensures jobs are removed when their parent application stops.\r\n\r\n<!-- tabs-open -->","ref":"readme.html#multi-register"},{"type":"extras","title":"Erlang - Home","doc":"```erlang\r\n{ok, _}= ecron:start_link(YourRegister),\r\necron:create(YourRegister, JobName, Spec, MFA, Options),\r\necron:delete(YourRegister, JobName).\r\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - Home","doc":"```elixir\r\n{:ok, _} = :ecron.start_link(YourRegister)\r\n:ecron.create(YourRegister, job_name, spec, mfa, options)\r\n:ecron.delete(YourRegister, job_name)\r\n```\r\n<!-- tabs-close -->\r\n\r\nAlternatively, use a supervisor:\r\n\r\n<!-- tabs-open -->","ref":"readme.html#elixir"},{"type":"extras","title":"Erlang - Home","doc":"[supervisor:child_spec/0](https://www.erlang.org/doc/apps/stdlib/supervisor.html#t:child_spec/0)\r\n```erlang\r\nYourRegister = your_register,\r\nChildren = [\r\n  #{\r\n        id => YourRegister,\r\n        start => {ecron, start_link, [YourRegister]},\r\n        restart => permanent,\r\n        shutdown => 1000,\r\n        type => worker\r\n    }\r\n]\r\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - Home","doc":"[supervisor:child_spec/1](https://hexdocs.pm/elixir/1.15.8/Supervisor.html#module-child_spec-1-function)\r\n```elixir\r\nchildren = [\r\n  worker(:ecron, [YourRegister], restart: :permanent)\r\n]\r\n\r\n```\r\n<!-- tabs-close -->\r\n\r\nAfter setup, use [`ecron:create/4`](https://hexdocs.pm/ecron/ecron.html#create/4) and [`ecron:delete/2`](https://hexdocs.pm/ecron/ecron.html#delete/2) to manage your jobs.","ref":"readme.html#elixir"},{"type":"extras","title":"Time Functions - Home","doc":"Ecron can manage recurring jobs. It also supports one-time tasks and time-based message delivery.\r\n\r\n[`ecron:send_after/3`](https://hexdocs.pm/ecron/ecron.html#send_after/2) Create a one-time timer that sends a message, just like [`erlang:send_after/3`](https://www.erlang.org/doc/apps/erts/erlang.html#send_after/3) but triggered with a crontab spec.\r\n<!-- tabs-open -->","ref":"readme.html#time-functions"},{"type":"extras","title":"Erlang - Home","doc":"```erlang\r\necron:send_after(\"*/4 * * * * *\", self(), hello_world),\r\nreceive Msg -> io:format(\"receive:~s~n\", [Msg]) after 5000 -> timeout end.\r\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - Home","doc":"```elixir\r\n:ecron.send_after(\"*/4 * * * * *\", self(), :hello_world)\r\nreceive do msg -> IO.puts(\"receive: #{msg}\") after 5000 -> :timeout end\r\n```\r\n<!-- tabs-close -->\r\n\r\nSends a message to a process repeatedly based on a crontab schedule.\r\n\r\n[`ecron:send_interval/3`](https://hexdocs.pm/ecron/ecron.html#send_interval/3) Create a repeating timer that sends a message, just like [`timer:send_interval/3`](https://www.erlang.org/doc/apps/stdlib/timer.html#send_interval/3) but triggered with a crontab spec.\r\n<!-- tabs-open -->","ref":"readme.html#elixir"},{"type":"extras","title":"Erlang - Home","doc":"```erlang\r\necron:send_interval(\"*/4 * * * * *\", self(), hello_world),\r\nLoop = fun(Loop) ->\r\n  receive\r\n    Msg -> \r\n      io:format(\"[~p] receive: ~s~n\", [erlang:time(), Msg]),\r\n      Loop(Loop)\r\n  after\r\n    5000 -> timeout\r\n  end\r\nend,\r\nLoop(Loop).\r\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - Home","doc":"```elixir\r\n:ecron.send_interval(\"*/4 * * * * *\", self(), :hello_world)\r\nloop = fn loop ->\r\n  receive do\r\n    msg -> \r\n      IO.puts(\"[#{Time.utc_now()}] receive: #{msg}\")\r\n      loop.(loop)\r\n  after\r\n    5000 -> :timeout\r\n  end\r\nend\r\nloop.(loop)\r\n```\r\n\r\n<!-- tabs-close -->","ref":"readme.html#elixir"},{"type":"extras","title":"Time Zone - Home","doc":"<!-- tabs-open -->","ref":"readme.html#time-zone"},{"type":"extras","title":"Erlang - Home","doc":"Configure ecron in your `sys.config` file with timezone:\r\n\r\n```erlang\r\n%% sys.config\r\n[\r\n   {ecron, [      \r\n      {time_zone, local} %% local or utc\r\n   ]}\r\n].\r\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - Home","doc":"```elixir\r\n# config/config.exs\r\nconfig :ecron,\r\n  time_zone: :local  # :local or :utc\r\n\r\n```\r\n<!-- tabs-close -->\r\n* When `time_zone` is set to `local` (default), ecron uses [calendar:local_time()](http://erlang.org/doc/man/calendar.html#local_time-0) to get the current datetime in your system's timezone\r\n* When `time_zone` is set to `utc`, ecron uses [calendar:universal_time()](http://erlang.org/doc/man/calendar.html#universal_time-0) to get the current datetime in UTC timezone","ref":"readme.html#elixir"},{"type":"extras","title":"Troubleshooting - Home","doc":"Ecron provides functions to assist with debugging at runtime:\r\n[`ecron:statistic/x ecron:parse_spec/2`](https://hexdocs.pm/ecron/ecron.html#debugging-testing)","ref":"readme.html#troubleshooting"},{"type":"extras","title":"Telemetry - Home","doc":"Ecron uses [Telemetry](https://github.com/beam-telemetry/telemetry) for instrumentation and logging.\r\nTelemetry is a metrics and instrumentation library for Erlang and Elixir applications \r\nthat is based on publishing events through a common interface and attaching handlers to handle those events. \r\nFor more information about the library itself, see its [README](https://github.com/beam-telemetry/telemetry).\r\n\r\nEcron logs all events by default.","ref":"readme.html#telemetry"},{"type":"extras","title":"Events - Home","doc":"|  Event      | measurements map key                       | metadata map key | log level\r\n| success     |run_microsecond,run_result,action_at        | name,mfa         | notice\r\n| activate    |action_at                                   | name,mfa         | notice\r\n| deactivate  |action_at                                   | name             | notice\r\n| delete      |action_at                                   | name             | notice\r\n| crashed     |run_microsecond,run_result,action_at        | name,mfa         | error\r\n| skipped     |job_last_pid,reason,action_at               | name,mfa         | error\r\n| aborted     |run_microsecond,action_at                   | name,mfa         | error\r\n| global,up   |quorum_size,good_nodes,bad_nodes,action_at  | self(node)       | alert\r\n| global,down |quorum_size,good_nodes,bad_nodes,action_at  | self(node)       | alert\r\n\r\nFor all failed events, refer to the documentation for [`ecron:statistic/2`](https://hexdocs.pm/ecron/ecron.html#statistic/2).\r\n\r\nYou can enable or disable logging via `log` configuration.\r\n<!-- tabs-open -->","ref":"readme.html#events"},{"type":"extras","title":"Erlang - Home","doc":"```erlang\r\n  %% sys.config\r\n  [\r\n   {ecron, [\r\n    %% none | all | alert | error | notice \r\n      {log_level, all}\r\n   ]}\r\n].\r\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - Home","doc":"```elixir\r\n# config/config.exs\r\nconfig :ecron,\r\n  # :none | :all | :alert | :error\r\n  log_level: :all\r\n```\r\n<!-- tabs-close -->\r\n- **all**: Captures all events.\r\n- **none**: Captures no events.\r\n- **alert**: Captures global up/down events.\r\n- **error**: Captures crashed, skipped, aborted, and global up/down events.\r\n\r\n>","ref":"readme.html#elixir"},{"type":"extras","title":"How? {: .info} - Home","doc":"> Use [logger:set_module_level(ecron_telemetry_logger, Log)](https://www.erlang.org/doc/apps/kernel/logger.html#set_module_level/2) to override the primary log level of Logger for log events originating from the ecron_telemetry_logger module.\r\n> This means that even if you set the primary log level to error, but the module log level is set to all, all ecron logs will be captured.","ref":"readme.html#how-info"},{"type":"extras","title":"Writing your own handler - Home","doc":"If you want custom logging control, you can create your own event handler. \r\nSee [`src/ecron_telemetry_logger.erl`](https://github.com/zhongwencool/ecron/blob/main/src/ecron_telemetry_logger.erl) as a reference implementation.","ref":"readme.html#writing-your-own-handler"},{"type":"extras","title":"Contributing - Home","doc":"To run property-based tests, common tests, and generate a coverage report with verbose output.\r\n```shell\r\n  $ rebar3 do proper -c, ct -c, cover -v\r\n```\r\nIt takes about 10-15 minutes.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Home","doc":"Ecron is released under the Apache-2.0 license. See the [license file](LICENSE).","ref":"readme.html#license"},{"type":"extras","title":"Crontab Spec","doc":"# Crontab Spec","ref":"crontab-spec.html"},{"type":"extras","title":"Basic Format - Crontab Spec","doc":"A cron expression consists of 5-6 fields representing time units:\n\n```\n# ┌────────────── second (optional)\n# │ ┌──────────── minute\n# │ │ ┌────────── hour\n# │ │ │ ┌──────── day of month\n# │ │ │ │ ┌────── month\n# │ │ │ │ │ ┌──── day of week\n# │ │ │ │ │ │\n# │ │ │ │ │ │\n# 0 * * * * *\n```","ref":"crontab-spec.html#basic-format"},{"type":"extras","title":"Field Values - Crontab Spec","doc":"Field | Required | Values | Special Characters\nSecond | No | 0-59 | `* / , -`\nMinute | Yes | 0-59 | `* / , -`\nHour | Yes | 0-23 | `* / , -`\nDay of Month | Yes | 1-31 | `* / , -`\nMonth | Yes | 1-12 or JAN-DEC | `* / , -`\nDay of Week | Yes | 0-6 or SUN-SAT | `* / , -`\n\n> **Note**: Month and Day-of-week values are case-insensitive.","ref":"crontab-spec.html#field-values"},{"type":"extras","title":"Special Characters - Crontab Spec","doc":"- `*` - Any value\n- `/` - Step values (e.g., `*/15` - every 15 units)\n- `,` - Value list (e.g., `1,3,5`)\n- `-` - Range (e.g., `1-5`)","ref":"crontab-spec.html#special-characters"},{"type":"extras","title":"Predefined Schedules - Crontab Spec","doc":"Expression | Description | Equivalent\n`@yearly` | Once a year (midnight, Jan 1) | `0 0 0 1 1 *`\n`@monthly` | Once a month (midnight, first day) | `0 0 0 1 * *`\n`@weekly` | Once a week (midnight, Sunday) | `0 0 0 * * 0`\n`@daily` | Once a day (midnight) | `0 0 0 * * *`\n`@hourly` | Once an hour | `0 0 * * * *`\n`@minutely` | Once a minute | `0 * * * * *`","ref":"crontab-spec.html#predefined-schedules"},{"type":"extras","title":"Fixed Intervals - Crontab Spec","doc":"For simpler scheduling needs, use `@every` with a duration:\n\n```erlang\n% Run every 30 minutes\n\"@every 30m\"\n\n% Run every 1 hour and 30 minutes\n\"@every 1h30m\"\n```\n\nDuration units: `d` (days), `h` (hours), `m` (minutes), `s` (seconds)","ref":"crontab-spec.html#fixed-intervals"},{"type":"extras","title":"Implementation","doc":"# Implementation\n\nEcron uses an efficient approach to manage job execution times and intervals:\n\n1. The top supervisor `ecron_sup` starts first.\n2. Then `ecron_sup` starts a child `ecron` (gen_server worker).\n3. `ecron` will look for the configuration `{local_jobs, Jobs}` at initialization.\n4. For each crontab job found, determine the next time in the future that each command must run.\n5. Place those commands in the ordered_set ETS with their `{NextCorrespondingTime, Name}` to run as the key.\n6. Enter `ecron`'s main loop:\n    * Examine the task entry at the head of the ETS, compute how far in the future it must run.\n    * Sleep for that period of time using the gen_server timeout feature.\n    * On awakening and after verifying the correct time, execute the task at the head of the ETS (spawn in background).\n    * Delete the old key in ETS.\n    * Determine the next time in the future to run this command and place it back in the ETS at that time value.\n    \nAdditionally, you can use `ecron:statistic(Name)` to see the job's latest 16 results and execution times.\n```\n    ecron_sup-------> ecron (registered as `ecron_local`)\n                       |\n                    |------| ets:new(timer, [ordered_set])\n                    | init | ets:insert(timer, [{{NextTriggeredTime,Name}...])\n                    |------|\n              |------->|\n              |     |------| {NextTriggeredTime,Name} = OldKey = ets:first(timer)\n              |     |      | sleep(NextTriggeredTime - current_time())\n              |     | loop | spawn a process to execute MFA\n              |     |      | ets:delete(timer, OldKey)\n              |     |------| ets:insert(timer, {{NewTriggeredTime,Name}...])\n              |<-------|\n```","ref":"implementation.html"},{"type":"extras","title":"Unique Job In Cluster","doc":"# Unique Job In Cluster","ref":"global.html"},{"type":"extras","title":"Precondition - Unique Job In Cluster","doc":"Global jobs depend on [global](http://erlang.org/doc/man/global.html), only allowed to be added statically.\n\n1. Fully Connected Cluster\n\n   Because it depends on `global`'s name registration service, \n   the name server must maintain a fully connected network.\n\n   For example, if node N1 connects to node N2 (which is already connected to N3), \n   the global name servers on nodes N1 and N3 ensure that N1 and N3 are also connected. \n   In other words, the command-line flag `-connect_all false` cannot be used.\n\n2. Same Global Configuration\n\n   All nodes' `global_quorum_size` and `global_jobs` must have the same value. \n   This ensures that the global task manager can transfer between nodes when the network splits.","ref":"global.html#precondition"},{"type":"extras","title":"Configuration - Unique Job In Cluster","doc":"#### global_jobs\n<!-- tabs-open -->","ref":"global.html#configuration"},{"type":"extras","title":"Erlang - Unique Job In Cluster","doc":"```erlang\n%% sys.config\n[\n   {ecron, [\n      {global_jobs, [\n         %% {JobName, CrontabSpec, {M, F, A}}\n         %% {JobName, CrontabSpec, {M, F, A}, PropListOpts}\n         %% CrontabSpec\n            %%  1. \"Minute Hour DayOfMonth Month DayOfWeek\"\n            %%  2. \"Second Minute Hour DayOfMonth Month DayOfWeek\"\n            %%  3. @yearly | @annually | @monthly | @weekly | @daily | @midnight | @hourly\n            %%  4. @every 1h2m3s\n                  \n         {basic, \"*/15 * * * *\", {io, format, [\"Runs on 0, 15, 30, 45 minutes~n\"]}}         \n     ]}     \n    }\n].\n```","ref":"global.html#erlang"},{"type":"extras","title":"Elixir - Unique Job In Cluster","doc":"Configure ecron in your `config.exs` file with job specifications:\n```elixir\n# config/config.exs\nconfig :ecron,  \n  local_jobs: [\n    # {job_name, crontab_spec, {module, function, args}}\n    # {job_name, crontab_spec, {module, function, args}, PropListOpts}\n    # CrontabSpec formats:\n    #  1. \"Minute Hour DayOfMonth Month DayOfWeek\"\n    #  2. \"Second Minute Hour DayOfMonth Month DayOfWeek\"\n    #  3. @yearly | @annually | @monthly | @weekly | @daily | @midnight | @hourly\n    #  4. @every 1h2m3s\n    \n    {:basic, \"*/15 * * * *\", {IO, :puts, [\"Runs on 0, 15, 30, 45 minutes\"]}}   \n  ] \n```\n<!-- tabs-close -->\n\nThe same format as `local_jobs`, default is `[]`. \nThis means only running local jobs without running global task manager and monitor processes.\n\n#### global_quorum_size\nThe ecron application lives on at least `global_quorum_size` nodes in the same cluster, which can be regarded as a healthy cluster. \nThe global task manager only runs on a healthy cluster.\n\nIf you want to guarantee always no more than **one** global task manager even when the cluster has a network split,\nyou should set it to **\"half plus one\"**. For example:\n\nRun on majority:\n   1. `ABC` 3 nodes in one cluster.\n   2. `global_quorum_size=2`.\n   3. (`ABC`) cluster splits into 2 parts (`AB`  =|=  `C`).\n   4. The global task manager would run on the `AB` cluster (`AB` is the healthy cluster now).\n   5. `C` node only runs local jobs without global jobs.\n\nRun on majority:\n   1. `ABC` 3 nodes in one cluster.\n   2. `global_quorum_size=2`.\n   3. (`ABC`) cluster splits into 3 parts (`A` =|= `B`  =|=  `C`).\n   4. Every node only runs local jobs without global jobs (all nodes are unhealthy).\n\nRun on every node if brain split:\n   1. `ABC` nodes in one cluster.    \n   2. `global_quorum_size=1`.\n   3. (`ABC`) cluster splits into 3 parts (`A` =|= `B`  =|=  `C`).\n   4. The global task manager would run on every node (we have three healthy clusters now).\n   5. But the global task manager only runs one in the same cluster.","ref":"global.html#elixir"},{"type":"extras","title":"Implementation - Unique Job In Cluster","doc":"1. The top supervisor `ecron_sup` starts first.\n2. Nothing will happen if the `global_jobs` is empty.\n3. `ecron_sup` would start_link `ecron_monitor` worker (gen_server) if `global_jobs` is not empty.\n4. `ecron_monitor` subscribes to node's up/down messages by [net_kernel:monitor_nodes(true)](http://erlang.org/doc/man/net_kernel.html#monitor_nodes-1) when it initializes.\n5. Enter `ecron_monitor` main loop:\n    * Checking if there are enough `ecron` processes in the cluster (`global_quorum_size`).\n    * Trying to terminate the global_job manager process when the cluster's `ecron` number is less than `global_quorum_size`.\n    * Otherwise, trying to start a global_job manager process. This gen_server registers by [global:register_name/2](http://erlang.org/doc/man/global.html#register_name-2).\n    * All the nodes are rushing to register this global jobs manager process; only one node will succeed, other nodes' `ecron_monitor` would link to this process if the process already exists.\n    * The `ecron_monitor` should receive a notification when a node goes down/up or the global_job manager has terminated.\n    * Go to step 5 again when notified.\n\n```\n     NodeA             NodeB             NodeC\n   supervisor        supervisor        supervisor\n       |                |     |            |\n    monitor          monitor  |          monitor\n         \\              |     |            /\n          \\           [link]  |           /\n           \\            |     |          /\n           [link]-----GlobalJob-----[link]\n``` \nNodeB has won the race and spawned the global manager process, other nodes' `ecron_monitor` only link to this manager.","ref":"global.html#implementation"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"1.1.0 - Changelog","doc":"","ref":"changelog.html#1-1-0"},{"type":"extras","title":"Features - Changelog","doc":"*   **Comprehensive Documentation**: Added extensive documentation using `?MODULEDOC` and `?DOC` macros to explain the module's purpose, functions, parameters, and return values. This includes:\r\n    *   Module-level documentation for `ecron`.\r\n    *   Detailed documentation for each exported function, including `create/3`, `create/4`, `send_after/3`, `send_interval/2`, `send_interval/3`, `send_interval/4`, `delete/1`, `delete/2`, `deactivate/1`, `activate/1`, `deactivate/2`, `activate/2`, `statistic/0`, `statistic/1`, `statistic/2`, `predict_datetime/2`, `predict_datetime/4`, `reload/0`, `reload/1`, and `parse_spec/2`.\r\n    *   Examples in both Erlang and Elixir for most functions.\r\n    *   Clear explanations of options, return values, and potential errors.\r\n\r\n*   **Deprecated Functions**: Marked older functions (`add/3`, `add/4`, `add/6`, `add/7`, `add_with_time/5`, `add_with_time/6`, `add_with_count/3`, `add_with_count/5`, `send_interval/5`, `send_interval/7`, `send_interval/8`) as deprecated, advising users to use `create/4` and `send_interval/4` instead.\r\n\r\n*   **Enhanced `create/4` Function**:\r\n    *   Added detailed explanations for the `Opts` parameter, including `register`, `start_time`, `end_time`, `singleton`, `max_count`, and `max_runtime_ms`.\r\n    *   Included a note about automatic job removal when `max_count` is reached.\r\n    *   Added a `TimeRange` info box explaining the constraints on `start_time` and `end_time`.\r\n    *   Added an example of handling invalid time configurations.\r\n\r\n*   **Enhanced `send_interval/4` Function**:\r\n    *   Added a note indicating that this is a repeatable timer and can be seen in `statistic/0` results.\r\n    *   Clarified that `ecron:delete/1` should be used to cancel the timer, not `erlang:cancel_timer/1`.\r\n\r\n*   **Enhanced `statistic/2` Function**:\r\n    *   Provided a detailed explanation of the `statistic()` return value, including descriptions of `ok`, `crashed`, `skipped`, and `aborted` states.\r\n\r\n*   **Time Zone Configuration**:\r\n    *   Added documentation for configuring the time zone using the `time_zone` application environment variable.\r\n    *   Explained the difference between `local` and `utc` time zone settings.\r\n\r\n*   **Telemetry Integration**:\r\n    *   Documented the use of Telemetry for instrumentation and logging.\r\n    *   Provided a table of Telemetry events, including `success`, `activate`, `deactivate`, `delete`, `crashed`, `skipped`, `aborted`, `global,up`, and `global,down`.\r\n    *   Explained how to enable or disable logging via the `log` configuration.\r\n    *   Provided guidance on writing custom Telemetry event handlers.\r\n\r\n*   **Internal Functions**:\r\n    *   Added `?DOC(false)` to internal functions to hide them from the generated documentation.","ref":"changelog.html#features"},{"type":"extras","title":"Changes - Changelog","doc":"*   **Refactored `add` Functions**: Replaced multiple `add` functions with a unified `create` function for better consistency and maintainability.\r\n\r\n*   **Improved Error Handling**: Added more specific error handling and return values for various functions.\r\n\r\n*   **Code Clarity**: Improved code readability by adding comments and using more descriptive variable names.","ref":"changelog.html#changes"},{"type":"extras","title":"Deprecations - Changelog","doc":"*   The following functions have been marked as deprecated:\r\n    *   `add/3`, `add/4`, `add/6`, `add/7`\r\n    *   `add_with_time/5`, `add_with_time/6`\r\n    *   `add_with_count/3`, `add_with_count/5`\r\n    *   `send_interval/5`, `send_interval/7`, `send_interval/8`","ref":"changelog.html#deprecations"},{"type":"extras","title":"Purpose - Changelog","doc":"These changes significantly enhance the usability and maintainability of the `ecron` library by providing comprehensive documentation, improving code structure, and integrating Telemetry for better monitoring and logging. The deprecation of older functions encourages users to adopt the more consistent and feature-rich `create` function.","ref":"changelog.html#purpose"},{"type":"extras","title":"1.0.2 - Changelog","doc":"- Use ex_doc to update documentation.","ref":"changelog.html#1-0-2"},{"type":"extras","title":"1.0.1 - Changelog","doc":"- Fix concurrent cron job execution timing issue.","ref":"changelog.html#1-0-1"},{"type":"extras","title":"1.0.0 - Changelog","doc":"Nothing changed, just update version from 0.6.1 to 1.0.0.","ref":"changelog.html#1-0-0"},{"type":"extras","title":"0.6.1 - Changelog","doc":"- fix some spec warning.\r\n- upgrade telemetry to 1.1.0","ref":"changelog.html#0-6-1"},{"type":"extras","title":"0.5.2 - Changelog","doc":"- redefine `[ecron global up/down]` telemetry event.","ref":"changelog.html#0-5-2"},{"type":"extras","title":"0.5.1 - Changelog","doc":"- Replace `cluster_quorum_size` by `global_quorum_size`.\r\n- Check if job's name is duplicate.","ref":"changelog.html#0-5-1"},{"type":"extras","title":"0.5.0 - Changelog","doc":"- Support global_jobs by `global`.\r\n- Add global/up/down telemetry metrics.","ref":"changelog.html#0-5-0"},{"type":"extras","title":"0.4.0 - Changelog","doc":"- Add `ecron:reload/0`.\r\n- Expose telemetry metrics.","ref":"changelog.html#0-4-0"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   Copyright 2019-2025, zhongwencool  .\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}